// ProcessDlg.cpp : implementation file
//

#include "stdafx.h"
#include "updalbum.h"
#include "ProcessDlg.h"
#include <setupapi.h>   // Includes the SetupAPI.
#include <memory>   // Includes the SetupAPI.
#include <wininet.h>
// Link with SetupAPI.Lib.
#pragma comment (lib, "setupapi.lib")


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CString CDAlbumSite::GetVersionStr(CInternetSession& session)
{
	if (!m_sVersion.IsEmpty())
		return m_sVersion;

	try 
	{
		// connect to delx.ca and get the version 
		auto_ptr<CStdioFile> pHTTP(session.OpenURL(SITE "/version.txt",1,INTERNET_FLAG_TRANSFER_ASCII | INTERNET_FLAG_RELOAD));
		
		CString s;
		pHTTP->ReadString(s);
		pHTTP->Close();

		m_sVersion=s;
		m_sVersion.TrimLeft();
		m_sVersion.TrimRight();

		return m_sVersion;
	}
	catch(CException *p)
	{
		m_pProcessDlg->ReportError("Unable to get current DAlbum version from " SITE ,p);
		p->Delete();
		return "";
	}

}
DWORD CDAlbumSite::GetBuild(CInternetSession& session)
{
	if (m_dwBuild!=0)
		return m_dwBuild;

	CString s=GetVersionStr(session);
	if (s.IsEmpty())
		return 0;

	LPCTSTR sz=_tcsrchr(s,'.');
	if (sz)
		m_dwBuild=_ttoi(sz+1);
	
	if (!m_dwBuild)
	{
		m_pProcessDlg->ReportError("Invalid version data retrieved from " SITE ,0);
		return 0;
	}
	return m_dwBuild;
}

CString CDAlbumSite::GetCABFile(CInternetSession& session)
{
	if (!m_sCABFile.IsEmpty())
		return m_sCABFile;
	
	// Create cab
	m_sCABFile=m_pProcessDlg->GetTempFileName();

	try 
	{
		m_pProcessDlg->SetStatus("Downloading latest DAlbum code");
		m_pProcessDlg->AddLog("Downloading latest DAlbum code...\r\n");
		// connect to delx.ca and get the version 
		auto_ptr<CStdioFile> pHTTP(session.OpenURL(SITE "/dalbum.cab",1,INTERNET_FLAG_TRANSFER_BINARY | INTERNET_FLAG_RESYNCHRONIZE));
		
		DWORD dw=0;
		CFile fo(m_sCABFile,CFile::modeCreate | CFile::modeWrite);
		do {
			BYTE buf[1024];
			
			UINT n=pHTTP->Read(buf,1024);
			if (n==0)
				break;
			fo.Write(buf,n);
			dw+=n;
		}while(1);
		fo.Close();

		if (dw<50000)
		{
			m_pProcessDlg->ReportError("Invalid DAlbum binaries were retrieved from " SITE ,0);
			return "";
		}
		m_pProcessDlg->SetStatus("DAlbum code downloaded successfully");
		m_pProcessDlg->AddLog("DAlbum code downloaded successfully...\r\n");
	}
	catch(CException *p)
	{
		m_pProcessDlg->ReportError("Unable to get current DAlbum version from " SITE ,p);
		p->Delete();
		return "";
	}
	return m_sCABFile;
}

/////////////////////////////////////////////////////////////////////////////
// CProcessDlg dialog


CProcessDlg::CProcessDlg(CPtrArray& f, BOOL bAll,CWnd* pParent /*=NULL*/)
	: CDialog(CProcessDlg::IDD, pParent), m_FTP(f)
{
	m_bAll=bAll;
	m_bRunning=0;
	//{{AFX_DATA_INIT(CProcessDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}

void CProcessDlg::SetStatus(LPCTSTR szStat)
{
	SetDlgItemText(IDC_STATUS,szStat);
	GetDlgItem(IDC_STATUS)->UpdateWindow();
	PumpMessages();
}
							

void CProcessDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CProcessDlg)
	DDX_Control(pDX, IDC_LOG, m_edLog);
	//}}AFX_DATA_MAP
}

CString CProcessDlg::GetTempFileName()
{
	TCHAR bufTemp[MAX_PATH],buf[MAX_PATH];
	if (GetTempPath(MAX_PATH,bufTemp) &&
		::GetTempFileName(bufTemp,"dalb_",0,buf))
		return buf;
	sprintf(buf,"dalb_%x.tmp",time(0));
	return buf;
}

void CProcessDlg::ReportError(LPCTSTR szError, CException *p)
{
	CString s = "Error: ";
	if (szError)
	{
		s += szError;
		s+= " : "  ;
	}
	if (p)
	{
		TCHAR buf[600]="Unknown error";
		p->GetErrorMessage(buf,600);
		s+=buf;
	}
	s+="\r\n";

	AfxMessageBox(s,MB_ICONERROR);
	AddLog(s);
}

void CProcessDlg::AddLog(LPCTSTR szStat)
{
	m_sLog+=szStat;
	m_edLog.SetWindowText(m_sLog);
	PumpMessages();
}

BEGIN_MESSAGE_MAP(CProcessDlg, CDialog)
	//{{AFX_MSG_MAP(CProcessDlg)
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CProcessDlg message handlers
struct TCurrentFile
{
	CFtpConnection *pFTP;
	CString			sTempFile;

	CString			sFilename;
	CString			sFolder;
	CString			sBaseFilename;
	CString			sCurrentDirectory;

	BOOL			bBinary;

	CProcessDlg		*pDlg;
};



UINT CALLBACK CabinetCallback(	PVOID Context,     // context used by the callback routine
							UINT Notification, // notification sent to callback routine
							UINT_PTR Param1,   // additional notification information
							UINT_PTR Param2    // additional notification information
						)
{
	LRESULT lRetVal = NO_ERROR;
	FILE_IN_CABINET_INFO *pInfo = NULL;
	FILEPATHS *pFilePaths = NULL;

	TCurrentFile *pc=(TCurrentFile *)Context;

	switch(Notification)
	{
	case SPFILENOTIFY_FILEINCABINET:
		pInfo = (FILE_IN_CABINET_INFO *)Param1;
		lRetVal = FILEOP_DOIT;  // Extract the file.
		{
			LPCSTR sz=strrchr(pInfo->NameInCabinet,'/');
			if (!sz)
				sz=strrchr(pInfo->NameInCabinet,'\\');

			if (sz)
			{
				pc->sFolder=CString(pInfo->NameInCabinet,sz-pInfo->NameInCabinet);
				pc->sBaseFilename=sz+1;
			}
			else
			{
				pc->sFolder="";
				pc->sBaseFilename=pInfo->NameInCabinet;
			}
			pc->sFilename=pInfo->NameInCabinet;

			sz=strrchr(pc->sBaseFilename,'.');
			CString sExt;
			if (sz)
				sExt=sz+1;
			sExt.MakeLower();

			pc->bBinary=true;
			if (sExt=="js" || sExt=="php" || sExt=="css" || sExt==".htpasswd" || sExt==".htaccess")
				pc->bBinary=false;
			if (pc->sBaseFilename=="version.txt")
				lRetVal = FILEOP_SKIP;  // Extract the file

		}

		lstrcpy(pInfo->FullTargetName, pc->sTempFile);

		break;

	case SPFILENOTIFY_FILEEXTRACTED:
		pFilePaths = (FILEPATHS *)Param1;

		{
			CString dir;

			dir=pc->sCurrentDirectory;
			if (dir.Right(1)=='/' ||
				dir.Right(1)=='\\')
				;
			else
				dir+='/';

			if (!pc->sFolder.IsEmpty())
			{
				dir+=pc->sFolder;
				dir.Replace("\\","/");
				BOOL bRes=pc->pFTP->CreateDirectory(dir);
				dir+="/";
			}


			CString sTarget=dir + pc->sBaseFilename;
			sTarget.Replace("\\","/");

			bool bSkip=false;
			if (pc->sBaseFilename.CompareNoCase("t_index.php")==0 ||
				pc->sBaseFilename.CompareNoCase("t_showimg.php")==0)
			{
				CFtpFileFind ff(pc->pFTP);
				if (ff.FindFile(sTarget))
					bSkip=true;
				ff.Close();
			}

			CString sOut;
			if (!bSkip)
				sOut.Format( "Uploading %s => %s ", LPCTSTR(pc->sFilename),  LPCTSTR(sTarget) );
			else
				sOut.Format( "Skipping %s", LPCTSTR(sTarget));
			pc->pDlg->SetStatus(sOut);
			if (pc->pDlg->m_bCancel)
			{
				lRetVal = NO_ERROR;
			}
			else if (!bSkip)
			{
				if (pc->pFTP->PutFile(pc->sTempFile,sTarget,pc->bBinary?FTP_TRANSFER_TYPE_BINARY:FTP_TRANSFER_TYPE_ASCII))
				{
					lRetVal = NO_ERROR;
				}
				else
				{
					lRetVal = GetLastError();
				}
			}
		}
		DeleteFile(pc->sTempFile);


		break;

	case SPFILENOTIFY_NEEDNEWCABINET: // Unexpected.
		lRetVal = NO_ERROR;
		break;
	}
	return lRetVal;
}

void CProcessDlg::OnTimer(UINT nIDEvent) 
{
	if (nIDEvent==1000)
	{
		CWaitCursor w;

		KillTimer(nIDEvent);

		m_bCancel=0;

		m_bRunning=0;
		CRunning run(GetDlgItem(IDCANCEL)->GetSafeHwnd(),&m_bRunning);

		LPSTR szOpt=AGENT;

		CInternetSession session(AGENT);
		
		SetStatus("Obtaining latest available DAlbum version from www.delx.ca/dalbum");
		AddLog("Obtaining latest available DAlbum version from www.delx.ca/dalbum...\r\n");
		
		CDAlbumSite st(this);
		CString sVer=st.GetVersionStr(session);
		DWORD dwBuild=st.GetBuild(session);
		if (sVer.IsEmpty() || dwBuild==0)
			return;

		SetStatus(CString("Latest DAlbum version is ") + sVer);
		AddLog(CString("Latest DAlbum version is ") + sVer + "\r\n");

		// 
		for (int nSite=0; nSite<m_FTP.GetSize();++nSite)
		{
			CFTPInfo *p=(CFTPInfo *)m_FTP[nSite];

			SetDlgItemText(IDC_SITE,p->m_sName +" (" + p->m_sServer + " as " + p->m_sUser + ")");
			PumpMessages();
			if (m_bCancel)
			{
				Canceled();
				return;
			}
			
			

			// 
			try {
				AddLog("=============================================\r\n");
				AddLog(CString(" Site: ")+p->m_sName+"\r\n");
				AddLog("=============================================\r\n");

				SetStatus(CString("Connecting to ") + p->m_sServer);

				auto_ptr<CFtpConnection> pFTP(session.GetFtpConnection(  p->m_sServer, p->m_sUser, p->m_sPassword, p->m_nPort, p->m_bPassive));

				SetStatus("Connected successfully. ");
				AddLog("Connected successfully.\r\n");
				
				CString sPath=p->m_sPath;

				if (!pFTP->SetCurrentDirectory(sPath))
				{
					ReportError(CString("Unable to set FTP directory to ")+sPath,0);
					return;
				}

				if (sPath.Right(1)=='/' ||
					sPath.Right(1)=='\\')
					;
				else
					sPath+='/';

				BOOL bOverwrite=TRUE;

				CString s;
					
				// Get version
				if (!m_bAll)
				{
					SetStatus("Obtaining DAlbum version number");
					try {
						auto_ptr<CInternetFile> pFile(pFTP->OpenFile("version.txt",GENERIC_READ,FTP_TRANSFER_TYPE_ASCII));	

						if (pFile->ReadString(s))
						{
							LPCTSTR sz=strrchr(s,'.');
							if (sz && _ttoi(sz+1)>=dwBuild)
								bOverwrite=FALSE;
						}
						pFile->Close();
						
					}
					catch (CException *e)
					{
						ReportError("version.txt not found - unable to determine site version",0);				
						e->Delete();
						return;
					}
				}

				if (!bOverwrite)
					AddLog(CString("Found DAlbum version ") + s + ". This version is up to date.\r\n");
				else
				{
					if (!m_bAll)
						AddLog(CString("Found DAlbum version ") + s + ". This version needs to be updated.\r\n");
					
					AddLog("Updating files...\r\n");

					TCurrentFile pf;
					pf.pDlg=this;
					pf.pFTP=pFTP.get();
					pf.sTempFile=GetTempFileName();
					pFTP->GetCurrentDirectory( pf.sCurrentDirectory) ;
					
					BOOL bIterate=SetupIterateCabinet(st.GetCABFile(session),0, (PSP_FILE_CALLBACK)CabinetCallback, &pf);
					if (!bIterate)
					{
						if (!m_bCancel)
							ReportError("DAlbum update failed",0);
						DeleteFile(pf.sTempFile);
						return;
					}
					else
					{
						pFTP->SetCurrentDirectory( pf.sCurrentDirectory) ;

						try {
							CFtpFileFind ff(pFTP.get());
							if (!ff.FindFile("custom.css"))
							{
								auto_ptr<CInternetFile> pFile(pFTP->OpenFile("custom.css",GENERIC_WRITE,FTP_TRANSFER_TYPE_ASCII));
								CString s="/* Insert your custom styles here */\n";
								pFile->Write(LPCSTR(s),s.GetLength());
								pFile->Close();
							}
							ff.Close();
						}
						catch (CException *p)
						{
							p->Delete();
						}

					
						AddLog("Done. Updating version information.\r\n");
						auto_ptr<CInternetFile> pFile(pFTP->OpenFile("version.txt",GENERIC_WRITE,FTP_TRANSFER_TYPE_ASCII));
						
						CString s=st.GetVersionStr(session);
						pFile->Write(LPCSTR(s),s.GetLength());
						pFile->Close();

						AddLog("\r\nSuccess. DAlbum updated successfully.\r\n");
					}
				}

				pFTP->Close();

			}
			catch (CException *p)
			{
				ReportError("An error occured accessing FTP",p);
				SetStatus("An error occured");
				return;
			}
		}
		AfxMessageBox("Operation completed successfully",MB_ICONINFORMATION);
		SetStatus("Operation completed successfully");

	}
	
	CDialog::OnTimer(nIDEvent);
}

BOOL CProcessDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	SetTimer(1000,50,0);	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CProcessDlg::OnCancel() 
{
	if (m_bRunning)
	{
		m_bCancel=TRUE;	
		return;
	}

	CDialog::OnCancel();
}
